# $Id: esr_image.rb,v 1.5 2006/10/21 07:47:38 nisyu Exp $

class EsrImageSetPro
	include BOS::PropMethods
	include BOS::SetupMethods
	
	Table_name = "bos_#{BOS::Prop[:site_name]}_esr_image_set_r"
	
	def initialize(p_id,p_mode='p')
		setup
		load_id p_id,p_mode
	end
	
	def setup_db
		@mydb.query("CREATE TABLE #{Table_name}(" +
			"image_set_r_id INT NOT NULL AUTO_INCREMENT PRIMARY KEY," +
			"sub_id INT NOT NULL," +
			"sub_type VARCHAR(1) NOT NULL," +
			"image_set_id INT NOT NULL," +
			"disp_order INT NOT NULL);")
	end
	
	def set_default
		{
			:image_set_r_id => 0,
			:sub_id => 0,
			:sub_type => 'p',
			:image_set_id => 0,
			:disp_order => 0 }
	end
	
	def load_id(p_id,p_mode)
	  @prop = @mydo.exp({
			:command => "s",
			:target  => Table_name,
			:cols    => set_default,
			:order   => "disp_order ASC",
			:where   => w("sub_id","sub_type"){|a,b| (a == p_id.to_i) & (b == p_mode)}})
		if @prop && @prop.size >= 1
		  @prop.each_with_index{|a,i|
        @prop[i][:image_set] = EsrImageSet.new(a[:image_set_id]) }
		end
	end
	
	def each
	  @prop.each{|im|
	    yield(im) }
	end
	
	def self.fastimage(p_id,p_mode,order)
	  fp = BOS::Rdb.exp({
			:command => "s",
			:target  => Table_name,
			:cols    => {:image_set_id => 0},
			:order   => "disp_order ASC",
			:where   => w("sub_id","sub_type","disp_order"){|a,b,c| (a == p_id.to_i) & (b == p_mode) & (c == order.to_i)}})[0]
		if fp
		  EsrImageSet.new(fp[:image_set_id])
		else
		  nil
		end
	end
	
end

class EsrImageSet
	include BOS::PropMethods
	include BOS::SetupMethods
	
	Table_name = "bos_#{BOS::Prop[:site_name]}_esr_image_set"
	
	def initialize(image_set_id)
		setup
		load_id image_set_id
	end
	
	def setup_db
		@mydb.query("CREATE TABLE #{Table_name}(" +
			"image_set_id INT NOT NULL AUTO_INCREMENT PRIMARY KEY," +
			"default_image_id INT," +
			"big_image_id INT," +
			"disp_flg VARCHAR(1) NOT NULL);")
	end
	
	def set_default
		{
			:image_set_id => 0,
			:default_image_id => 0,
			:big_image_id => 0,
			:disp_flg => "p"}
	end
	
	def load_id(image_set_id)
	  @prop = @mydo.exp({
			:command => "s",
			:target  => Table_name,
			:cols    => set_default,
			:where   => w("image_set_id","disp_flg"){|a,b| (a == image_set_id.to_i) & (b == 'p')}})[0]
		
		@default_image = EsrImage.new(@prop[:default_image_id])
		@big_image = EsrImage.new(@prop[:big_image_id])
	end
	
	def get_image_urls
	  [@default_image.get_image_url, @big_image.get_image_url]
	end
end

class EsrImage
	include BOS::PropMethods
	include BOS::SetupMethods

	Table_name = "bos_#{BOS::Prop[:site_name]}_esr_images"
	ImageFolder = 'http://e-28.jp/image/' || '/image/'
	
	def initialize(image_id)
		setup
		load_id image_id
	end
	
	def setup_db
		@mydb.query("CREATE TABLE #{Table_name}(" +
			"image_id INT NOT NULL AUTO_INCREMENT PRIMARY KEY," +
			"ex_str VARCHAR(8) NOT NULL," +
			"info_str VARCHAR(255) NOT NULL," +
			"up_date DATETIME NOT NULL);")
	end
	
	def set_default
		{
			:image_id => 0,
			:ex_str => "",
			:info_str => "",
			:up_date => DateTime.now }
	end
	
	def load_id(image_id)
	  @prop = @mydo.exp({
			:command => "s",
			:target  => Table_name,
			:cols    => set_default,
			:where   => w("image_id"){|a| a == image_id.to_i}})[0]
	end
	
	def get_image_url
	  if @prop
      "%s%06d\.%s" % [ImageFolder,@prop[:image_id],@prop[:ex_str]]
    else
      nil
    end
	end
	
end

