# $Id: esr_item.rb,v 1.11 2006/10/19 05:28:56 nisyu Exp $

class EsrItem
	include BOS::PropMethods
	include BOS::SetupMethods
	attr_reader :mate_list
	
	Table_name = "bos_#{BOS::Prop[:site_name]}_esr_items"
	Table_name_o = "bos_#{BOS::Prop[:site_name]}_esr_item_options"
	Table_name_r = "bos_#{BOS::Prop[:site_name]}_esr_item_r"
	
	def initialize(item_id)
		@mate_list = nil
		setup
		load_id(item_id)
	end
	
	def setup_db
		@mydb.query("CREATE TABLE #{Table_name}(" +
			"item_id INT NOT NULL AUTO_INCREMENT PRIMARY KEY," +
			"item_name TEXT NOT NULL," +
			"item_code VARCHAR(255) NOT NULL," +
			"inventory INT NOT NULL," +
			"r_inventory INT NOT NULL," +
			"cost INT NOT NULL," +
			"price INT NOT NULL," +
			"normal_price INT NOT NULL," +
			"pretax INT NOT NULL," +
			"disp_start_date DATETIME," +
			"disp_end_date DATETIME," +
			"disp_flg CHAR(1) NOT NULL);")
		@mydb.query("CREATE TABLE #{Table_name_o}(" +
			"item_option_id INT NOT NULL AUTO_INCREMENT PRIMARY KEY," +
			"self_item_id INT NOT NULL," +
			"option_name VARCHAR(255) NOT NULL," +
			"inventory INT NOT NULL," +
			"r_inventory INT NOT NULL," +
			"price INT NOT NULL," +
			"normal_price INT NOT NULL);")
		@mydb.query("CREATE TABLE #{Table_name_r}(" +
			"item_r_id INT NOT NULL AUTO_INCREMENT PRIMARY KEY," +
			"self_item_id INT NOT NULL," +
			"item_option_id INT NOT NULL," +
			"sub_material_id INT NOT NULL," +
			"quantity INT NOT NULL);")
	end
	
	def set_default(tfl=nil)
		case tfl
		when 'r'
			{
				:item_r_id => 0,
				:self_item_id => 0,
				:item_option_id => 0,
				:sub_material_id => 0,
				:quantity => 1 }
		when 'o'
			{
				:item_option_id => 0,
				:self_item_id => 0,
				:option_name => 'new option',
				:inventory => 0,
				:r_inventory => 0,
				:price => 0,
				:normal_price => 0 }
		else
			{
				:item_id => 0,
				:item_name => 'new item',
				:item_code => '',
				:inventory => 0,
				:r_inventory => 0,
				:cost => 0,
				:price => 0,
				:normal_price => 0,
				:pretax => 1,
				:disp_start_date => DateTime.now,
				:disp_end_date   => DateTime.parse("9999/01/01"),
				:disp_flg => '' }
		end
	end
	
	def load_id(pid)
		return nil unless pid
		@prop = @mydo.exp({
			:command => "s",
			:target  => Table_name,
			:cols    => set_default,
			:where   => w('item_id'){|a| a == pid } })[0]
		@prop = set_default unless @prop
		load_mate_list
	end
	
	def load_mate_list(pid=nil)
		pid ||= @prop[:item_id]
		if pid
			mlt = @mydo.exp({
				:command => "s",
				:target  => Table_name_r,
				:cols    => set_default('r'),
				:where   => w('self_item_id'){|a| a == pid } })
			if mlt.size > 0
		    @mate_list = []
				mlt.each do |m|
					@mate_list << BOS::EsrMaterial.new(m[:sub_material_id],m[:quantity])
				end
			end
		end
	end
	
	def add_mate(mate_id,ct=1,option_id=0)
	  return nil unless mate_id
	  return nil unless @prop[:item_id]
		mlt = @mydo.exp({
			:command => "d",
			:target  => Table_name_r,
			:where   => w('self_item_id','item_option_id','sub_material_id'){|a,b,c| (a == @prop[:item_id]) & (b == option_id) & (c == mate_id) } })
		if ct >= 1
  		mlt = @mydo.exp({
  			:command => "i",
  			:target  => Table_name_r,
  			:cols    => {
  				:self_item_id => @prop[:item_id] ,
  				:item_option_id => option_id ,
  				:sub_material_id => mate_id ,
  				:quantity => ct }})
  	end
		load_mate_list
	end
	
	def clear_mate(option_id=0)
	  return nil unless @prop[:item_id]
	  if @mate_list
	    tmate = @mate_list
	    tmate.each{|m| add_mate(m[:material_id],0,option_id) }
	  end
	end
	
	def chk_inventory_mode
		ret = ''
		ret << 'm' if @mate_list
		ret << 'o' if @option_list
		ret
	end
	
	def get_cost
		ret = 0
		if chk_inventory_mode.include?('m')
			@mate_list.each{|m| ret += m.get_cost }
		else
			ret = @prop[:cost]
		end
		ret
	end
	
	def get_inventory(r=false)
		ret = 0
		if chk_inventory_mode.include?('m')
		  ret = []
			@mate_list.each{|m| ret << m.get_inventory(r) }
			ret = ret.sort[0]
		else
			ret = r ? @prop[:r_inventory] : @prop[:inventory]
		end
		ret
	end
	
	def set_inventory(qt, r=false)
		retct = nil
		if chk_inventory_mode.include?('m')
			if r
				retct = get_inventory
			else
				if (get_inventory + qt) >= 0
					@mate_list.each{|m|
						m.set_inventory(qt) }
				else
					retct = get_inventory
				end
			end
		else
			minv = r ? :r_inventory : :inventory
			if (@prop[:inventory] + qt) >= 0
				@prop[minv] = @prop[minv] + qt
				@prop[:inventory] = @prop[:inventory] + qt if r
				commit
			else
				retct = @prop[minv]
			end
		end
		return retct
	end
	
	def set_inventory_to_r(qt)
		retct = nil
		rt_diff = get_inventory(true) - get_inventory
		if rt_diff >= qt
			if chk_inventory_mode.include?('m')
				@mate_list.each{|m|
					m.set_inventory_to_r(qt) }
			else
				@prop[:r_inventory] = @prop[:r_inventory] - qt
				commit
			end
		else
			retct = rt_diff
		end
		retct
	end
	
	def get_list(pg=1, pc=10, sc="", sb=nil, wh=nil)
		pg = pg || 1
		pc = pc || 10
		@mydo.exp({
			:command => "s",
			:target  => Table_name,
			:cols    => set_default,
			:order   => sb ? "#{sb[:key]} #{sb[:mode]}" : "item_id DESC",
			:page    => pg,
			:where   => wh ? wh : nil,
			:pagesize => pc })
	end
	
	def get_plist
	  EsrProduct.new(0).get_item_plist @prop[:item_id]
	end
	
	def get_mate_plist(mate_id)
	  deh = {:item_id=>0 , :item_name => ""}
		@mydo.exp({
			:command => "s",
			:target  => lj(Table_name,Table_name_r){|a,b| a[:item_id] == b[:self_item_id]},
			:cols    => deh,
			:where   => w('sub_material_id'){|a| a == mate_id},
			:order   => "item_id DESC" })
	end
	
	def commit
		if @prop[:item_id] == 0 then
			@prop.delete(:item_id)
			@prop[:item_id] = @mydo.exp({
				:command => "i",
				:target  => Table_name,
				:cols    => @prop })[:insid]
		else
			lct = @mydo.exp({
				:command => "u",
				:target  => Table_name,
				:cols    => @prop,
				:where   => w("item_id"){|a| a == @prop[:item_id]}})
		end
		
		@prop[:item_id]
	end

end
